# Generated by Django 4.1.2 on 2022-12-26 06:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0002_remove_keywords_type_keywords_description_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='keywords',
            name='avalable',
            field=models.CharField(blank=True, help_text='Value of site-wide variable that scripts can reference', max_length=10, null=True),
        ),
    ]



# Generated by Django 4.1.2 on 2023-01-06 13:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('snort', '0003_snortruleviewarray_htmlid_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='snortrule',
            name='deleted',
            field=models.BooleanField(default=False),
        ),
    ]



from django.db import models

# Create your models here.


class Pcap(models.Model):
    name = models.CharField(max_length=100)
    description = models.TextField(max_length=256)
    extra = models.TextField(max_length=256,blank=True)
    pcap_file = models.FileField(upload_to="pacp_repo/", null=True)
    date = models.DateTimeField(auto_now=True)
    rule_to_validate = models.ForeignKey('snort.SnortRule', blank=True, null=True, on_delete=models.SET_NULL)
    objects = models.Manager()

    def __str__(self):
        return self.name

class white_Pcap(models.Model):
    name = models.CharField(max_length=100)
    description = models.TextField(max_length=256)
    extra = models.TextField(max_length=256,blank=True)
    pcap_file = models.FileField(upload_to="pacp_repo/", null=True)
    date = models.DateTimeField(auto_now=True)
    rule_to_validate = models.ForeignKey('snort.SnortRule', blank=True, null=True, on_delete=models.SET_NULL)
    objects = models.Manager()

    def __str__(self):
        return self.name





class snort_template():

    def __init__(self, protocol="tcp", src_port="any", dst_port="any", msg="{sig_group} {sig_name} - {sig_content}",
                 file_data=False, content=True, case_sensitive=False, sig_type="string", tag=True):
        tagged_content = 'content:"{sig_content}"; ' if content else ""
        tagged_file_data ="file_data; " if file_data else ""
        taggged_case_sensitive = "" if case_sensitive else "nocase; "
        metadata = 'team "{writer_team}", employee "{sig_writer}", group "{sig_group}", name "{sig_name}",' \
                   ' treatment "{main_doc}", keywords "None", date "{date}", document "{sig_ref}",' \
                   ' description "{sig_desc}"'
        tagged_tag = " tag:session,10,packets;" if tag else ""
        self.rule_string = f'alert {protocol} {src_port} any <> any {dst_port} (msg: "{msg};' \
                           f' {tagged_file_data}{tagged_content}{taggged_case_sensitive}' \
                           'sid:{sid}; rev:1;gid:1000000; metadata:type' + f' "{sig_type}", {metadata};)'

    def get_rule(self, sig_group, sig_name, sig_content, writer_team, sig_writer, main_doc, cur_date, sig_ref, sig_desc, sid=0):
        sig_content = sig_content.replace("\"", "\\\"")
        sig_desc = sig_desc.replace("\"", "\\\"")
        return self.rule_string.format(**{"sig_group": sig_group, "sig_name": sig_name, "sig_content": sig_content,
                                        "writer_team": writer_team, "sig_writer": sig_writer, "main_doc": main_doc,
                                        "date": cur_date, "sig_ref": sig_ref, "sig_desc": sig_desc, "sid": f"{sid}"})


class Preprocessor(snort_template):
    def __init__(self):
        super().__init__(protocol="ip", file_data=True, content=True)


class ipfull(snort_template):
    def __init__(self):
        super().__init__(src_port="any", msg="IP_RULE:{2}:IP_RULE", content="", nocase="", rule_type="ip")


snort_type_to_template = {"type a": ipfull,
                          "Preprocessor": Preprocessor}
EMPTY_TYPES = ["type b"]


types_list = list(zip(["a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p"][:len(snort_type_to_template.keys()) +1], snort_type_to_template.keys()))


from django.contrib import admin
from django import forms
import os
from .models import Pcap, white_Pcap


class PcapAdminForm(forms.ModelForm):
    class Meta:
        model = Pcap
        fields = "__all__"

    def clean_pcap_file(self):
        try:
            if not verify_legal_pcap(self.cleaned_data.get("pcap_file")):
                raise Exception(f"illegal pcap file")
        except Exception as e:
            raise forms.ValidationError(f"cant validate pcap file: {e}")

        old_pcap_file = self.initial.get("pcap_file").url if hasattr(self.initial.get("pcap_file"), "url") else ""
        delete_old = False
        if os.path.exists(old_pcap_file):
            if hasattr(self.cleaned_data.get("pcap_file"), "url"):
                if self.initial.get("pcap_file").url != self.cleaned_data.get("pcap_file").url:
                    delete_old = True
            else:
                delete_old = True
        if delete_old:
            os.remove(self.initial.get("pcap_file").url)
        return self.cleaned_data.get("pcap_file")


@admin.register(white_Pcap)
class SnortRuleAdmin(admin.ModelAdmin):
    def validate(self, request, obj: white_Pcap):
        # test saved rule vs pcap
        print("validate button pushed", obj.name)
    # validate.label = "validate"  # optional
    # validate.color = "green"
    # validate.short_description = "Submit this article"  # optional

    # change_actions = ('validate', )
    # changelist_actions = ('validate',)

    list_display = ("name", "description", "pcap_file", "rule_to_validate", "date")
    search_fields = ("name", "description", "pcap_file", "rule_to_validate")
    # form = SnortRuleAdminForm


@admin.register(Pcap)
class SnortRuleAdmin(admin.ModelAdmin):
    def validate(self, request, obj: Pcap):
        # test saved rule vs pcap
        print("validate button pushed", obj.name)
    # validate.label = "validate"  # optional
    # validate.color = "green"
    # validate.short_description = "Submit this article"  # optional

    # change_actions = ('validate', )
    # changelist_actions = ('validate',)

    list_display = ("name", "description", "pcap_file", "rule_to_validate", "date")
    search_fields = ("name", "description", "pcap_file", "rule_to_validate")
    # form = SnortRuleAdminForm


def verify_legal_pcap(filename):
    import dpkt
    counter = 0

    for ts, pkt in dpkt.pcap.Reader(open(filename, 'br')):

        counter += 1
        eth = dpkt.ethernet.Ethernet(pkt)
        if eth.type != dpkt.ethernet.ETH_TYPE_IP:
            continue

    if not counter:
        return False
    return True



"""snort_web_master URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/4.1/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.contrib import admin
from django.urls import path, include
from django.shortcuts import redirect
from snort.views import get_rule, get_rule_keys, build_rule_keyword_to_rule, build_rule_rule_to_keywords, favico
from django.conf import settings
from django.conf.urls.static import static

admin.site.site_header = 'snort web master'

urlpatterns = [
    path("favicon.ico", favico),
    path("get_rule_update/<int:rule_id>/", get_rule),
    path("get_rule_keywords/<int:rule_id>/", get_rule_keys),
    path('advanced_filters/', include("advanced_filters.urls")),
    path("build_rule/keyword_to_rule", build_rule_keyword_to_rule),
    path("build_rule/rule_to_keywords", build_rule_rule_to_keywords),
    path('admin/', admin.site.urls),
    path('', lambda request: redirect('admin/', permanent=True)),
]+ static("static/", document_root=settings.STATIC_ROOT) + static("/", document_root=settings.BASE_DIR)



import suricataparser
from django.http.response import HttpResponse, JsonResponse
from snort.models import SnortRule, SnortRuleViewArray
import json
import os
from django.conf import settings
# Create your views here.

def get_rule_keys(request, rule_id=None):
    rule_keywordss = SnortRuleViewArray.objects.filter(**{"snortId": rule_id})
    results = {"data": []}
    for rule_key in rule_keywordss:
        results["data"].append({"htmlId": rule_key.htmlId, "value": rule_key.value, "typeOfItem": rule_key.typeOfItem,
                        "locationX": rule_key.locationX, "locationY": rule_key.locationY})
    return JsonResponse(results)


def get_rule(request, rule_id=None):
    full_rule = SnortRule.objects.get(**{"id": rule_id}).content

    return HttpResponse(full_rule)


def build_rule_keyword_to_rule(request, full_rule=""):
    if not full_rule:
        full_rule = json.loads(request.body.decode()).get("fule_rule")
    resppnse = {"data": []}
    rule_parsed = suricataparser.parse_rule(full_rule)
    build_keyword_dict(resppnse, rule_parsed)
    return JsonResponse(resppnse)


def build_keyword_dict(resppnse, rule_parsed):
    rule_keywordss = [build_keyword_item("action", rule_parsed.action),
                      build_keyword_item("protocol", rule_parsed.header.split(" ")[0]),
                      build_keyword_item("srcipallow",
                                         "!" if rule_parsed.header.split(" ")[1].startswith("!") else "-----"),
                      build_keyword_item("srcip", rule_parsed.header.split(" ")[1][1:] if rule_parsed.header.split(" ")[
                          1].startswith("!") else rule_parsed.header.split(" ")[1], item_type="input"),
                      build_keyword_item("srcportallow",
                                         "!" if rule_parsed.header.split(" ")[2].startswith("!") else "-----"),
                      build_keyword_item("srcport",
                                         rule_parsed.header.split(" ")[2][1:] if rule_parsed.header.split(" ")[
                                             2].startswith("!") else rule_parsed.header.split(" ")[2],
                                         item_type="input"),
                      build_keyword_item("direction", rule_parsed.header.split(" ")[3]),
                      build_keyword_item("dstipallow",
                                         "!" if rule_parsed.header.split(" ")[4].startswith("!") else "-----"),
                      build_keyword_item("dstip", rule_parsed.header.split(" ")[4][1:] if rule_parsed.header.split(" ")[
                          4].startswith("!") else rule_parsed.header.split(" ")[4], item_type="input"),
                      build_keyword_item("dstportallow",
                                         "!" if rule_parsed.header.split(" ")[5].startswith("!") else "-----"),
                      build_keyword_item("dstport",
                                         rule_parsed.header.split(" ")[5][1:] if rule_parsed.header.split(" ")[
                                             5].startswith("!") else rule_parsed.header.split(" ")[5],
                                         item_type="input"),
                      ]
    i = 0
    op_num = 0
    for op in rule_parsed.options:
        if op.name in ["msg", "sid"]:
            resppnse[op.name] = op.value
            i += 1
            continue
        if op.name == "metadata":
            for item in op.value.data:
                for meta_value in ["group ", "name ", "treatment ", "document ", "description "]:
                    if item.strip("'").strip().startswith(meta_value):
                        resppnse["metadata_" + meta_value.strip()] = item.strip("'").strip().replace(meta_value, "")
            continue
        rule_keywordss.append(build_keyword_item("keyword_selection" + str(op_num), op.name, x=op_num, y=0))

        if op.value:
            if op.value.startswith("!"):
                rule_keywordss.append(
                    build_keyword_item(f"keyword{str(op_num)}" + "-not", "!", x=op_num, y=0,
                                       item_type="input"))
                op.value = op.value[1:]
            rule_keywordss.append(
                build_keyword_item(f"keyword_selection{str(op_num)}" + "-data", op.value.strip('"'), x=op_num, y=0,
                                   item_type="input"))
        op_num += 1
        i += 1
    for rule_key in rule_keywordss:
        resppnse["data"].append(
            {"htmlId": rule_key["htmlId"], "value": rule_key["value"], "typeOfItem": rule_key["typeOfItem"],
             "locationX": rule_key["locationX"], "locationY": rule_key["locationY"]})


def build_keyword_item(my_id, value, item_type="select", x=0, y=0):
    return {"htmlId": my_id, "value": value, "typeOfItem": item_type,
            "locationX": x, "locationY": y}


# build_rule_keyword_to_rule(None, SnortRule.objects.get(**{"id": 5}).content)
def build_rule_rule_to_keywords(request, rule_keywords=None):
    resppnse = {"fule_rule": ""}
    if not rule_keywords:
        rule_keywords = {}
    return JsonResponse(resppnse)

def favico(request):
    image_data = open(os.path.join(settings.BASE_DIR, "favicon.ico"), "rb").read()
    return HttpResponse(image_data, content_type="image/png")


# Generated by Django 4.1.2 on 2022-12-19 18:24

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pcaps', '0002_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='pcap',
            old_name='rule_to_validate',
            new_name='rule_to_validate2',
        ),
        migrations.RenameField(
            model_name='white_pcap',
            old_name='rule_to_validate',
            new_name='rule_to_validate2',
        ),
    ]



# Generated by Django 4.1.2 on 2022-12-19 18:20

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('settings', '0001_initial'),
        ('pcaps', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SnortRule',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False)),
                ('active', models.BooleanField(default=False)),
                ('admin_locked', models.BooleanField(default=False)),
                ('user', models.CharField(blank=True, default=0, max_length=100)),
                ('name', models.CharField(max_length=100)),
                ('type', models.TextField(choices=[('a', 'type a'), ('b', 'Preprocessor')], max_length=30)),
                ('content', models.TextField(max_length=2048)),
                ('description', models.TextField(max_length=256)),
                ('extra', models.TextField(blank=True, max_length=256)),
                ('location', models.CharField(max_length=256)),
                ('request_ref', models.CharField(blank=True, max_length=12)),
                ('main_ref', models.CharField(blank=True, max_length=12)),
                ('date', models.DateTimeField(auto_now=True)),
                ('template', models.TextField(blank=True, max_length=512)),
                ('group', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='settings.attackgroup')),
                ('pcap_legal_check', models.ManyToManyField(blank=True, related_name='pcap_legal_check', to='pcaps.white_pcap')),
                ('pcap_sanity_check', models.ManyToManyField(blank=True, related_name='pcap_sanity_check', to='pcaps.pcap')),
            ],
            options={
                'ordering': ('name', 'type', 'date'),
            },
        ),
    ]



# Generated by Django 4.1.2 on 2023-01-06 06:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('snort', '0002_snortruleviewarray'),
    ]

    operations = [
        migrations.AddField(
            model_name='snortruleviewarray',
            name='htmlId',
            field=models.CharField(blank=True, max_length=20),
        ),
        migrations.AlterField(
            model_name='snortruleviewarray',
            name='typeOfItem',
            field=models.CharField(blank=True, max_length=20),
        ),
    ]



"""
Django settings for snort_web_master project.

Generated by 'django-admin startproject' using Django 4.1.2.

For more information on this file, see
https://docs.djangoproject.com/en/4.1/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/4.1/ref/settings/
"""
import os
from pathlib import Path
import mimetypes

mimetypes.add_type("text/javascript", ".js", True)
# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/4.1/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = os.environ.get("SECRET_KEY", 'django-insecure-6imn8m6=okyge)ghoygg&rz3zzei45f5!5%#*mmy1e*ekj*cq5')

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = int(os.environ.get("DEBUG", default=0))

ALLOWED_HOSTS = os.environ.get("DJANGO_ALLOWED_HOSTS", "").split(" ") or ["*"]

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        'LOCATION': 'unique-snowflake',
    }
}
# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    "snort_web_master",
    'snort',
    'pcaps',
    "django_object_actions",
    "settings",
    "advanced_filters",
    'import_export'

]
IMPORT_EXPORT_USE_TRANSACTIONS = True
MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    "snort_web_master.middleware.no_cache.NoCacheControl",
]

ROOT_URLCONF = 'snort_web_master.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR,'templates')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'snort_web_master.wsgi.application'


# Database
# https://docs.djangoproject.com/en/4.1/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": os.environ.get("SQL_ENGINE", "django.db.backends.sqlite3"),
        "NAME": os.environ.get("SQL_DATABASE", BASE_DIR / "db.sqlite3"),
        "USER": os.environ.get("SQL_USER", "user"),
        "PASSWORD": os.environ.get("SQL_PASSWORD", "password"),
        "HOST": os.environ.get("SQL_HOST", "localhost"),
        "PORT": os.environ.get("SQL_PORT", "5432"),
    }
}


# Password validation
# https://docs.djangoproject.com/en/4.1/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/4.1/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/4.1/howto/static-files/
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)

STATIC_URL = "/static/"
STATIC_ROOT = BASE_DIR / "static"
STATICFILES_DIRS = [os.path.join(BASE_DIR, 'static')]

# Default primary key field type
# https://docs.djangoproject.com/en/4.1/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'
# LDAP_SERVERS = [
#     {
#         'host': 'ldap.forumsys.com',
#         'port': 389,
#         'use_ssl': False,
#         'get_info': 'NONE',
#     }
# ]
# # LDAP_ENGINE = 'OpenLDAP'
# # https://coderbook.com/@marcus/how-to-add-ldap-and-active-directory-authentication-to-django/
# LDAP_BIND_USER = "cn=read-only-admin,dc=example,dc=com"
# LDAP_BIND_PASSWORD = "password"
# from ldap3 import NTLM
# LDAP_AUTHENTICATION = NTLM
# LDAP_SEARCH_BASE = "dc=example,dc=com"
# LDAP_USER_SEARCH_FILTER = "(&(sAMAccountName=%s)(objectClass=user))"
# LDAP_USER_MODEL_USERNAME = "id"
# LDAP_USE_LDAP_GROUP = False
# LDAP_ATTRIBUTES_MAP = {
#     'username': 'sAMAccountName',
#     'first_name': 'givenName',
#     'last_name': 'sn',
#     'email': 'mail',
# }
# LDAP_GROUPS_MAP = {
#     'mathematicians': "cn=read-only-admin,dc=domain,dc=com",
# }



from django.db import models

# Create your models here.
class Setting(models.Model):
    """
    Model for site-wide settings.
    """
    name = models.CharField(max_length=200, help_text="Name of site-wide variable")
    value = models.CharField(null=True, blank=True, max_length=100, help_text="Value of site-wide variable that scripts can reference - must be valid JSON")

    def __unicode__(self):
        return self.name


class attackGroup(models.Model):
    name = models.CharField(max_length=100)
    description = models.TextField(max_length=256)
    objects = models.Manager()

    def __str__(self):
        return self.name

# Create your models here.
class keywords(models.Model):
    """
    Model for site-wide settings.
    """
    name = models.CharField(max_length=200, help_text="Name of site-wide variable")
    stage = models.CharField(null=True, blank=True, max_length=100, help_text="Value of site-wide variable that scripts can reference")
    description = models.CharField(null=True, blank=True, max_length=100, help_text="Value of site-wide variable that scripts can reference")
    # negation = models.CharField(null=True, blank=True, max_length=100,
    #                                help_text="Value of site-wide variable that scripts can reference")
    options = models.CharField(null=True, blank=True, max_length=100,
                                help_text="Value of site-wide variable that scripts can reference")
    avalable = models.CharField(null=True, blank=True, max_length=10,
                                help_text="Value of site-wide variable that scripts can reference")
    def __unicode__(self):
        return self.name


from django.db import models
from settings.models import attackGroup
from .snort_templates import types_list

class SnortRule(models.Model):
    id = models.AutoField(primary_key=True)
    group = models.ForeignKey(attackGroup, blank=True, null=True, on_delete=models.SET_NULL)
    active = models.BooleanField(default=False)
    admin_locked = models.BooleanField(default=False)
    user = models.CharField(max_length=100, blank=True, default=0)
    name = models.CharField(max_length=100)
    type = models.TextField(max_length=30, choices=types_list)
    content = models.TextField(max_length=2048)
    description = models.TextField(max_length=256)
    extra = models.TextField(max_length=256, blank=True)
    location = models.CharField(max_length=256)
    request_ref = models.CharField(max_length=12, blank=True)
    main_ref = models.CharField(max_length=12, blank=True)
    date = models.DateTimeField(auto_now=True)
    is_template = models.BooleanField(default=False)
    pcap_sanity_check = models.ManyToManyField("pcaps.Pcap", related_name='pcap_sanity_check', blank=True)
    pcap_legal_check = models.ManyToManyField("pcaps.white_Pcap", related_name='pcap_legal_check', blank=True)
    objects = models.Manager()
    deleted = models.BooleanField(default=False)

    def delete(self):
        self.active = False
        self.deleted = True
        self.save()
    class Meta:
        ordering = ("name", "type", "date")


class SnortRuleViewArray(models.Model):
    id = models.AutoField(primary_key=True)
    snortId = models.ForeignKey('snort.SnortRule', blank=True, null=True, on_delete=models.SET_NULL)
    typeOfItem = models.CharField(max_length=20, blank=True)
    locationX = models.IntegerField()
    locationY = models.IntegerField()
    value = models.CharField(max_length=256, blank=True)
    htmlId = models.CharField(max_length=50, blank=True)

def delete_rule_from_s3(rule_id):
    # todo: implement
    # check if exists
    # delete
    pass
def save_rule_to_s3(rule_id, rule_content):
    # upsert rule
    pass


# Generated by Django 4.1.2 on 2022-12-19 18:20

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('snort', '0001_initial'),
        ('pcaps', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='white_pcap',
            name='rule_to_validate',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='snort.snortrule'),
        ),
        migrations.AddField(
            model_name='pcap',
            name='rule_to_validate',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='snort.snortrule'),
        ),
    ]



import copy
import os
import csv
import time
from io import StringIO
from django.contrib import messages
from functools import partial
from import_export.admin import ImportExportModelAdmin
from django import forms
from .models import SnortRule, SnortRuleViewArray, save_rule_to_s3, delete_rule_from_s3
from .snort_templates import types_list
from .parser import Parser
from django.utils.encoding import smart_str
from django.http.response import HttpResponse, HttpResponseRedirect
from django.utils.html import mark_safe
from django.db import transaction
import suricataparser
from snort.views import build_keyword_dict
# Register your models here.
from django.contrib import admin
from django_object_actions import DjangoObjectActions
import subprocess
from settings.models import Setting, keywords, attackGroup
from django.shortcuts import render
from pcaps.admin import verify_legal_pcap
from advanced_filters.admin import AdminAdvancedFiltersMixin
from django.core.files.storage import default_storage
from django.core.files.base import ContentFile
from django.conf import settings
from datetime import datetime


class StoreAdminForm(forms.ModelForm):
    ## add an extra field:
    upfile = forms.FileField()

    class Meta:
        model = SnortRule
        fields = "__all__"

    def clean(self):
        cleaned_data = super(StoreAdminForm, self).clean()

        if "upfile" in self.changed_data:
            ### file validation on file type etc here ..
            ## file is valid:
            ## next lines deal with the InMemoryUploadedFile Type
            path = settings.MEDIA_ROOT.joinpath("___tmp___")
            tmp = default_storage.save(path, ContentFile(cleaned_data["upfile"].read()))

            ## ...

BASE_FIELDS = [
    "id", "active", "is_template", "deleted", "admin_locked", 'name', "request_ref", "main_ref", "description",
    "extra", "location", "user"]
FILTER_FIELDS = ("active", "is_template", "deleted", "admin_locked")
ADVANCE_FILTER_FIELDS = tuple(i for i in BASE_FIELDS + ["content", ("pcap_sanity_check__name", "pcap_sanity_check_name"), ("pcap_legal_check__name", "pcap_legal_check_name"),("group_name", "group_name")])
FIELDS = [
    "id", "content", "active", "is_template", "deleted", "admin_locked", 'name', "request_ref", "main_ref", "snort_builder", "description",
    "group", "extra", "location", "user", 'pcap_sanity_check', "pcap_legal_check", ]
SEARCH_FIELDS = tuple(i for i in BASE_FIELDS + ["content", "pcap_sanity_check__name", "pcap_legal_check__name", "group__name"])
BASE_BUILDER_KEY = ("action", "protocol", "srcipallow", "srcip", "srcportallow", "srcport", "direction", "dstipallow",
                    "dstportallow", "dstport")

INPUT_TYPE = ("srcip" , "srcport", "dstip", "dstport")
from django.core.cache import cache
# todo: upload unmanaged rule file

class SnortRuleAdminForm(forms.ModelForm):
    def clean_user(self):
        return getattr(self.current_user, self.current_user.USERNAME_FIELD)

    def clean_date(self):
        return self.cleaned_data["date"]

    def clean_type(self):
        if not dict(types_list).get(self.cleaned_data.get("type")):
            raise forms.ValidationError("cant find type, did you forgot it? or forgot to add type to db", code=404)
        return self.cleaned_data.get("type")

    def clean_content(self):
        try:
            parser = Parser(self.data["content"])
            parser.parse_header()
            parser.parse_options()
        except Exception as e:
            raise forms.ValidationError(e)

        return self.data["content"]

    def clean_location(self):
        try:
            if os.path.dirname(self.cleaned_data["location"]) != "":
                os.makedirs(os.path.dirname(self.cleaned_data["location"]), exist_ok=True)
            os.makedirs(os.path.dirname(self.cleaned_data["location"]), exist_ok=True)
            with open(self.cleaned_data["location"], "w") as rule_file:
                rule_file.write(self.cleaned_data["content"])
        except Exception as e:
            forms.ValidationError(e)
        return self.cleaned_data["location"]

    def clean_is_template(self):
        if not self.cleaned_data.get("is_template"):
            self.cleaned_data["active"] = False
        return self.cleaned_data.get("is_template")

    def clean_pcap_sanity_check(self):
        # return self.cleaned_data.get("pcap_validation")
        if not self.cleaned_data.get("pcap_sanity_check"):
            if Setting.objects.get(**{"name": "FORCE_SANITY_CHECK"}).value == "False":
                return self.cleaned_data["pcap_sanity_check"]
            elif Setting.objects.get(**{"name": "FORCE_SANITY_CHECK"}).value == "True":
                raise forms.ValidationError(
                    f"no pcap provided fof sanity check, plase add pcap or edit setting(FORCE_SANITY_CHECK)")
            else:
                raise forms.ValidationError(
                    f"bad configuration setting (FORCE_SANITY_CHECK), pleas edit setting(FORCE_SANITY_CHECK) must be True or False")
        cur_rule = SnortRule()
        cur_rule.content = self.data.get("content")
        cur_rule.location = self.data.get("location")
        cur_rule.group = self.instance.group
        cur_rule.id = self.data.get("id")
        cur_rule.main_ref = self.data.get("main_ref")
        cur_rule.name = self.data.get("name")
        cur_rule.type = self.data.get("type")
        cur_rule.user = self.data.get("user")
        cur_rule.request_ref = self.data.get("request_ref")

        validate_pcap_snort(self.cleaned_data.get("pcap_sanity_check"), cur_rule)
        return self.cleaned_data["pcap_sanity_check"]

    # only admin can activate admin locked rule
    def clean_pcap_legal_check(self):
        # return self.cleaned_data.get("pcap_validation")

        if not self.cleaned_data.get("pcap_legal_check"):
            if Setting.objects.get(**{"name": "FORCE_LEGAL_CHECK"}).value == "False":
                return self.cleaned_data["pcap_legal_check"]
            elif Setting.objects.get(**{"name": "FORCE_LEGAL_CHECK"}).value == "True":
                raise forms.ValidationError(
                    f"no pcap provided fof sanity check, plase add pcap or edit setting(FORCE_LEGAL_CHECK)")
            else:
                raise forms.ValidationError(
                    f"bad configuration setting (FORCE_LEGAL_CHECK), pleas edit setting(FORCE_LEGAL_CHECK) must be True or False")

        cur_rule = SnortRule()
        cur_rule.content = self.data.get("content")
        cur_rule.location = self.data.get("location")
        cur_rule.group = self.data.get("group")
        cur_rule.id = self.data.get("id")
        cur_rule.main_ref = self.data.get("main_ref")
        cur_rule.name = self.data.get("name")
        cur_rule.type = self.data.get("type")
        cur_rule.user = self.data.get("user")
        cur_rule.request_ref = self.data.get("request_ref")

        count = validate_pcap_snort(self.cleaned_data.get("pcap_legal_check"), cur_rule)
        max_allowd = self.cleaned_data["MAX_MATCH_ALLOWD"]
        if int(count) > max_allowd:
            self.cleaned_data["admin_locked"] = True
            self.instance.admin_locked = True
            self.instance.save()
            if self.cleaned_data["active"] == True:
                if not self.current_user.is_staff and not self.current_user.is_superuser:
                    raise forms.ValidationError(
                        f"rule is admin locked due to hige number of validations {count}, please contact admin or fix rule\n all changed of an admin locked rull must be approved by admin")
        else:
            self.cleaned_data["admin_locked"] = False
            self.instance.admin_locked = False
            self.instance.save()

        return self.cleaned_data["pcap_legal_check"]

    def clean_active(self):
        if self.instance.active:
            return self.cleaned_data["active"]
        locked = False
        if self.cleaned_data.get("admin_locked") is None:
            locked = self.instance.admin_locked
        else:
            locked = self.cleaned_data.get("admin_locked")
        if self.cleaned_data["active"] and locked:
            if not self.current_user.is_staff and not self.current_user.is_superuser:
                raise forms.ValidationError(
                    f"rule is admin locked, please contact admin", code=403)
        return self.cleaned_data["active"]

    @transaction.atomic
    def clean(self):
        try:
            self.instance.user = self.clean_user()
            self.instance.content = self.clean_content()
        except Exception as e:
            self.add_error(None, e)
        rule_keys = []
        self.instance.deleted = False
        if not self.instance.pk and not self.errors:
            self.instance.save()
        # SnortRuleViewArray.objects.filter(snortId=None).delete()
        if not self.errors:
            SnortRuleViewArray.objects.filter(snortId=self.instance.id).delete()
        for key, value in self.data.items():
            if key in FIELDS + ['csrfmiddlewaretoken', "_save"]:
                continue
            item_type = "select"
            location_x = 0
            location_y = 0
            if "keyword_selection" in key:
                location_x = 0
                try:
                    index = key.index("-")
                except:
                    index = len(key)
                location_y = int(key[len("keyword_selection"):index])
            elif "keyword" in key:
                try:
                    index = key.index("-", key.index("-") + 1)
                except:
                    index = len(key)
                if key[key.index("-") + 1:index] == "not":
                    location_x = 0
                    location_y = 0
                else:
                    location_x = int(key[key.index("-") + 1:index])
                    location_y = int(key[len("keyword"):key.index("-")])
            if "-data" in key or key in INPUT_TYPE:
                item_type = "input"
            rule_keys.append(SnortRuleViewArray(snortId=self.instance,
                               typeOfItem=item_type,
                               locationX=location_x,
                               locationY=location_y,
                               value=value,
                               htmlId=key))
        if not self.errors:
            cache.set(self.instance.id, rule_keys)
            for key in rule_keys:
                key.save()
        else:
            cache.set(self.instance.id, rule_keys)
            return
        if self.cleaned_data.get("active"):
            save_rule_to_s3(self.instance.id, self.instance.content)
            pass
            # todo: save to s3
        else:
            delete_rule_from_s3(self.instance.id)
            # todo: make sure it is not on prod


def validate_pcap_snort(pcaps, rule):
    stdout = b""

    if not rule.location:
        import re
        rule.location = re.sub(r'[-\s]+', '-', re.sub(r'[^\w\s-]', '',
                                                      rule.name)
                               .strip()
                               .lower())

    with open(rule.location + ".tmp", "w") as rule_file:
        rule_file.write(rule.content)
    failed = True
    for pcap in pcaps:
        try:
            if not verify_legal_pcap("/app/{pcap.pcap_file}"):
                raise Exception(f"illegal pcap file")
            if not os.path.exists(f"/app/{pcap.pcap_file}"):
                raise Exception(f"cant find file /app/{pcap.pcap_file}")
            stdout, stderr = subprocess.Popen(
                ["/home/snorty/snort3/bin/snort", "-R", rule.location + ".tmp", "-r", f"/app/{pcap.pcap_file}", "-A",
                 "fast"], stdout=subprocess.PIPE,
                stderr=subprocess.PIPE).communicate()
            if stdout and not stderr:
                if b"total_alerts: " in stdout:
                    return stdout.split(b"total_alerts: ")[1].split(b"\n")[0]
                else:
                    return 0
        except Exception as e:
            raise forms.ValidationError(f"could not validate rule on {pcap.pcap_file}: {e}", code=405)
    if failed:
        raise Exception("no rules was chosen")
    return stdout


@admin.register(SnortRule)
class SnortRuleAdmin(DjangoObjectActions, AdminAdvancedFiltersMixin, ImportExportModelAdmin, admin.ModelAdmin):
    list_filter = FILTER_FIELDS  # simple list filters

    # specify which fields can be selected in the advanced filter
    # creation form
    advanced_filter_fields = ADVANCE_FILTER_FIELDS
    change_actions = ('clone_rule',)
    # changelist_actions = ('load_template',)
    fields = FIELDS
    filter_horizontal = ('pcap_sanity_check', "pcap_legal_check")
    list_display_links = ("name",)
    list_display = ("id", "user", "active", "name", "group", "description", "content", "date", "is_template")
    search_fields = SEARCH_FIELDS
    form = SnortRuleAdminForm
    actions = ['make_published']

    def export_action(self, request):
        return self.export_data(SnortRule.objects.all())

    @transaction.atomic
    def import_action(self, request):
        errors = False
        if request.method == 'POST':
            snort_rules_to_save = []
            snort_rules_options_to_save = {}
            try:
                csv_file = request.FILES['myfile']
                if not csv_file.name.endswith('.csv'):
                    messages.error(request, 'File is not CSV type')
                    return HttpResponseRedirect("/admin/snort/snortrule/import/")
                # if file is too large, return
                if csv_file.multiple_chunks():
                    messages.error(request, "Uploaded file is too big (%.2f MB)." % (csv_file.size / (1000 * 1000),))
                    return HttpResponseRedirect("/admin/snort/snortrule/import/")

                file_data = csv.DictReader(StringIO(csv_file.read().decode("utf-8")))
                # loop over the lines and save them in db. If error , store as string and then display
                for item in file_data:
                    snort_rule = SnortRule()
                    snort_rule.active = item["Active"]
                    snort_rule.deleted = item["Deleted"]
                    snort_rule.user = getattr(request.user, request.user.USERNAME_FIELD)
                    snort_rule.description = item["Description"]
                    snort_rule.extra = item["Extra"]
                    if item.get("Group"):
                        snort_rule.group = attackGroup.objects.get(name=item["Group"])
                    snort_rule.name = item["Name"]
                    if item.get("Id"):
                        snort_rule.id = item["Id"]
                    else:
                        temp_id = str(time.time())
                        snort_rule.id = "temp " + temp_id
                    snort_rule.content = item["Rule"]

                    try:
                        resppnse = {"data": []}
                        try:
                            rule_parsed = suricataparser.parse_rule(item["Rule"])
                        except:
                            raise Exception("bad rule format")
                        build_keyword_dict(resppnse, rule_parsed)
                        for item_data in resppnse["data"]:
                            temp_id = snort_rule.id
                            snort_rules_options_to_save[temp_id] = []
                            snort_rules_options_to_save[temp_id].append(SnortRuleViewArray(**item_data))
                        for op in rule_parsed.options:
                            if op.name == "msg":
                                if snort_rule.group:
                                    op.value = snort_rule.group.name + " "
                                else:
                                    op.value = ""
                                if snort_rule.name:
                                    op.value += snort_rule.name
                                continue
                            if op.name == "sid":
                                op.value = snort_rule.id
                                continue
                            if op.name == "metadata":
                                new_value = []
                                user_applyed = False
                                for item_metadata in op.value.data:
                                    if item_metadata.strip("'").strip().startswith("group "):
                                        if snort_rule.group:
                                            new_value.append(f"group {snort_rule.group.name}")
                                            continue
                                    if item_metadata.strip("'").strip().startswith("name "):
                                        new_value.append(f"name {snort_rule.name}")
                                        continue
                                    if item_metadata.strip("'").strip().startswith("description "):
                                        new_value.append(f"description {snort_rule.description}")
                                        continue
                                    if item_metadata.strip("'").strip().startswith("employee "):
                                        new_value.append(f"employee {snort_rule.user}")
                                        user_applyed = False
                                        continue
                                    if item_metadata.strip("'").strip().startswith("document "):
                                        snort_rule.main_ref = item_metadata.strip("'").strip().replace("document ", "")
                                    if item_metadata.strip("'").strip().startswith("treatment "):
                                        snort_rule.request_ref = item_metadata.strip("'").strip().replace("document ", "")
                                    new_value.append(item_metadata)
                                if not user_applyed:
                                    new_value.append(f"employee {snort_rule.user}")
                                op.value.data = new_value
                                continue
                        snort_rule.content = rule_parsed
                        snort_rules_to_save.append(snort_rule)
                    except Exception as e:
                        import traceback
                        traceback.print_exc()
                        errors = True
                        messages.error(request, f"Unable to load rule {item['Name']}. {repr(e)}")
                        pass

            except Exception as e:
                errors = True
                messages.error(request, "Unable to upload file. " + repr(e))
            if not errors:
                for rule in snort_rules_to_save:
                    rule_id = rule.id
                    if rule_id.startswith("temp "):
                        rule.id = None
                        rule_parsed = rule.content
                        for op in rule_parsed.options:
                            if op.name == "sid":
                                op.value = 0
                                break
                        rule.content = ""
                        rule.save()
                        for op in rule_parsed.options:
                            if op.name == "sid":
                                op.value = rule.id
                                break
                    rule.content = rule_parsed.build_rule()
                    rule.save()
                    if rule.active:
                        save_rule_to_s3(rule.id, rule.content)
                    else:
                        delete_rule_from_s3(rule.id)
                    SnortRuleViewArray.objects.filter(snortId=rule.id).delete()
                    for attr in snort_rules_options_to_save[rule_id]:
                        attr.snortId = rule
                        attr.save()
                return HttpResponseRedirect("/admin/snort/snortrule/")
            return HttpResponseRedirect("/admin/snort/snortrule/import/")

        return render(request, 'html/import.html')

    @admin.action(description='export selected snort to csv')
    def make_published(self, request, queryset):
        return self.export_data(queryset)

    def export_data(self, queryset):
        response = HttpResponse(
            content_type='application/force-download')  # mimetype is replaced by content_type for django 1.7
        response['Content-Disposition'] = 'attachment; filename=%s' % smart_str(f"SnortRule-{datetime.now()}.csv")
        str_content = "Active,Date,Deleted,Description,Extra,Group,Name,Id,User,Rule\n"
        for snort_item in queryset:
            content = snort_item.content.replace('"', "'")
            my_list = [snort_item.active, snort_item.date, snort_item.deleted, snort_item.description, snort_item.extra,
                       snort_item.group, snort_item.name, snort_item.pk, snort_item.user, content]
            for item in my_list:
                if isinstance(item, bool):
                    str_content += str(item) + ","
                    continue
                if not item:
                    str_content += ','
                    continue
                str_content += '"' + str(item) + '",'
                continue
            str_content = str_content[:-1] + "\n"
        response.content = smart_str(str_content)
        # It's usually a good idea to set the 'Content-Length' header too.
        # You can also set any other required headers: Cache-Control, etc.
        return response

    def get_actions(self, request):
        actions = super().get_actions(request)
        if 'delete_selected' in actions:
            del actions['delete_selected']
        return actions

    def snort_builder(self, obj):
        set_rule = cache.get(obj.id)
        if not set_rule:
            set_rule = SnortRuleViewArray.objects.filter(snortId=obj.id)
            cache.set(obj.id, set_rule)
        else:
            cache.set(obj.id, [])
        context = copy.deepcopy(self.context)
        context["build_items"] = set_rule
        snort_buider_section = self.snort_buider_section(context).content.decode("utf-8")
        return mark_safe(self.full_rule_js.content.decode("utf-8") + snort_buider_section)

    def get_form(self, request, *args, **kwargs):
        form = super(SnortRuleAdmin, self).get_form(request, **kwargs)
        form.current_user = request.user
        context = {"actions": keywords.objects.filter(stage="action", avalable="True"),
                   "protocols": keywords.objects.filter(stage="protocol", avalable="True")}
        self.snort_buider_section = partial(render, request, "html/snortBuilder.html")
        self.full_rule_js = render(request, "html/full_rule.html")
        self.context = context
        return form

    @transaction.atomic
    def clone_rule(self, request, obj: SnortRule):
        new_snort = SnortRule.objects.get(pk=obj.id)
        new_snort.pk = None
        new_snort.is_template = False
        new_snort.active = False
        new_snort.deleted = False
        new_snort.user = getattr(request.user, request.user.USERNAME_FIELD)
        new_snort.save()
        for keyword in SnortRuleViewArray.objects.filter(snortId=obj.id):
            keyword.pk = None
            keyword.snortId = new_snort
            keyword.save()

        return HttpResponseRedirect(f"/snort/snortrule/{new_snort.pk}/change")

    clone_rule.label = "clone_rule"  # optional

    def get_readonly_fields(self, request, obj=None):
        if obj and (obj.is_template or obj.admin_locked):
            read_only_fields = (
            "id", "active", 'location', "user", "admin_locked", "snort_builder", "deleted")
        else:
            read_only_fields = ("id", 'location', "user", "admin_locked", "snort_builder", "deleted")

        return read_only_fields

    # readonly_fields = ("id", 'location', "user", "admin_locked", "full_rule", "snort_builder", "deleted")
    clone_rule.short_description = "clone rule to a new rule"  # optional



# Generated by Django 4.1.2 on 2022-12-19 18:20

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Pcap',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('description', models.TextField(max_length=256)),
                ('extra', models.TextField(blank=True, max_length=256)),
                ('pcap_file', models.FileField(null=True, upload_to='pacp_repo/')),
                ('date', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name='white_Pcap',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('description', models.TextField(max_length=256)),
                ('extra', models.TextField(blank=True, max_length=256)),
                ('pcap_file', models.FileField(null=True, upload_to='pacp_repo/')),
                ('date', models.DateTimeField(auto_now=True)),
            ],
        ),
    ]




import re
import ipaddress
import collections
from typing import Tuple, List, Dict, Any

try:
    from .dicts import Dicts
except ImportError:
    from dicts import Dicts


class Parser(object):
    '''
    this will take an array of lines and parse it and hand
    back a dictionary
    NOTE: if you pass an invalid rule to the parser,
    it will a raise ValueError.
    '''

    def __init__(self, rule):
        self.dicts = Dicts()
        self.rule = rule
        self.header = self.parse_header()
        self.options = self.parse_options()
        self.validate_options(self.options)
        self.data = {"header": self.header, "options": self.options}
        self.all = self.data

    def __iter__(self):
        yield self.data

    def __getitem__(self, key):
        if key == 'all':
            return self.data
        else:
            return self.data[key]

    @staticmethod
    def actions(action: str) -> str:
        actions = {
            "alert",
            "log",
            "pass",
            "activate",
            "dynamic",
            "drop",
            "reject",
            "sdrop"
        }

        if action in actions:
            return action
        else:
            msg = "Invalid action specified %s" % action
            raise ValueError(msg)

    @staticmethod
    def proto(proto: str) -> str:
        protos = {
            "tcp",
            "udp",
            "icmp",
            "ip"
        }

        if proto.lower() in protos:
            return proto
        else:
            msg = "Unsupported Protocol %s " % proto
            raise ValueError(msg)

    @staticmethod
    def __ip_to_tuple(ip: str) -> Tuple:
        if ip.startswith("!"):
            ip = ip.lstrip("!")
            return False, ip
        else:
            return True, ip

    def __form_ip_list(self, ip_list: str) -> List:
        ip_list = ip_list.split(",")
        ips = []
        for ip in ip_list:
            ips.append(self.__ip_to_tuple(ip))
        return ips

    def __flatten_ip(self, ip):
        list_deny = True
        if ip.startswith("!"):
            list_deny = False
            ip = ip.lstrip("!")
        _ip_list = []
        _not_nest = True
        ip = re.sub(r'^\[|\]$', '', ip)
        ip = re.sub(r'"', '', ip)
        if re.search(r"(\[.*\])", ip):
            _not_nest = False
            nest = re.split(r",(!?\[.*\])", ip)
            nest = filter(None, nest)
            # unnest from _ip_list
            _return_ips = []
            for item in nest:
                if re.match(r"^\[|^!\[", item):
                    nested = self.__flatten_ip(item)
                    _return_ips.append(nested)
                    continue
                else:
                    _ip_list = self. __form_ip_list(item)
                    for _ip in _ip_list:
                        _return_ips.append(_ip)
            return list_deny, _return_ips
        if _not_nest:
            _ip_list = self. __form_ip_list(ip)
            return list_deny, _ip_list

    def __validate_ip(self, ips):
        variables = {
            "$EXTERNAL_NET",
            "$HTTP_SERVERS",
            "$INTERNAL_NET",
            "$SQL_SERVERS",
            "$SMTP_SERVERS",
            "$DNS_SERVERS",
            "$HOME_NET",
            "HOME_NET",
            "any"
        }

        for item in ips:

            if isinstance(item, bool):
                pass

            if isinstance(item, list):
                for ip in item:
                    self.__validate_ip(ip)

            if isinstance(item, str):
                if item not in variables:
                    if "/" in item:
                        ipaddress.ip_network(item)
                    else:
                        ipaddress.ip_address(item)
        return True

    def ip(self, ip):
        if isinstance(ip, str):
            ip = ip.strip('"')
            if re.search(r",", ip):
                item = self.__flatten_ip(ip)
                ip = item
            else:
                ip = self.__ip_to_tuple(ip)
            valid = self.__validate_ip(ip)
            if valid:
                return ip
            else:
                raise ValueError("Unvalid ip or variable: %s" % ip)

    @staticmethod
    def port(port):
        variables = {"any", "$HTTP_PORTS"}

        # is the source marked as not
        if port.startswith("!"):
            if_not = False
            port = port.strip("!")
        else:
            if_not = True
        # is it a list ?
        # if it is, then make it a list from the string
        """
        Snort allows for ports marked between
        square brackets and are used to define lists
        correct:
        >> [80:443,!90,8080]
        >> ![80:443]
        >> [!80:443]
        """

        if port.startswith("["):
            if port.endswith("]"):
                port = port[1:-1].split(",")
            else:
                raise ValueError("Port list is malformed")

        if isinstance(port, list):
            ports = []

            for item in port:
                not_range = True

                if ":" in item:
                    # Checking later on if port is [prt:] or [:prt]
                    open_range = False
                    items = item.split(":", 1)
                    message = ''
                    for prt in items:
                        message = "Port range is malformed %s" % item
                        prt = prt.lstrip("!")
                        if not prt:
                            open_range = True
                            continue

                        try:
                            prt = int(prt)
                        except:
                            raise ValueError(message)

                        if prt < 0 or prt > 65535:
                            raise ValueError(message)

                    for index, value in enumerate(items):
                        value = value.lstrip("!")
                        items[index] = value

                    if not open_range:
                        try:
                            a = int(items[-1])
                            b = int(items[0])
                        except:
                            raise ValueError(message)
                        if a - b < 0:
                            raise ValueError(message)
                    not_range = False

                port_not = True
                if re.search("^!", item):
                    port_not = False
                    item = item.strip("!")
                if not_range:
                    if item.lower() or item in variables:
                        ports.append((port_not, item))
                        continue
                    try:
                        prt = int(item)
                        if prt < 0 or prt > 65535:
                            raise ValueError("Port is out of range {}".format(item))
                    except ValueError:
                        raise ValueError("Unknown port {}".format(item))
                ports.append((port_not, item))

            return if_not, ports

        if isinstance(port, str):
            """
            Parsing ports like: :8080, 80:, 80:443
            and passes all variables ex: $HTTP
            ranges do not accept denial (!)
            """
            if port or port.lower() in variables or re.search(r"^\$+", port):
                return if_not, port

            if re.search(":", port):
                message = "Port is out of range %s" % port
                ports = port.split(":")
                for portl in ports:
                    portl.lstrip("!")
                    if not portl:
                        continue
                    if portl or portl.lower() in variables:
                        continue
                    try:
                        portl = int(portl)

                    except ValueError:
                        raise ValueError(message)
                    if portl < 0 or portl > 65535:
                        raise ValueError(message)

                return if_not, port

            """
            Parsing a single port
            single port accepts denial.
            """
            try:
                if not int(port) > 65535 or int(port) < 0:
                    return if_not, port

                if int(port) > 65535 or int(port) < 0:
                    raise ValueError

            except:
                msg = "Unknown port: \"%s\" " % port
                raise ValueError(msg)
        else:
            message = "Unknown port \"%s\"" % port
            raise ValueError(message)

    def destination(self, dst):
        destinations = {"->": "to_dst",
                        "<>": "bi_direct"}

        if dst in destinations:
            return dst
        else:
            msg = "Invalid destination variable %s" % dst
            raise ValueError(msg)

    def get_header(self):
        if re.match(r'(^[a-z|A-Z].+?)?(\(.+;\)|;\s\))', self.rule.lstrip()):
            header = self.rule.split('(', 1)
            return header[0]
        else:
            msg = 'Error in syntax, check if rule'\
                  'has been closed properly %s ' % self.rule
            raise SyntaxError(msg)

    @staticmethod
    def remove_leading_spaces(string: str) -> str:
        return string.strip()

    def get_options(self):
        options = "{}".format(self.rule.split('(', 1)[-1].lstrip().rstrip())
        if not options.endswith(")"):
            raise ValueError("Snort rule options is not closed properly, "
                             "you have a syntax error")

        op_list = list()

        value = ""
        option = ""
        last_char = ""

        for char in options.rstrip(")"):
            if char != ";":
                value = value + char
                option = option + char

            if char == ";" and last_char != "\\":
                op_list.append(option.strip())
                value = option = ""

            last_char = char

        return op_list

    def parse_header(self):
        """
        OrderedDict([('action', 'alert'), ('proto', 'tcp'), ('source', \
        (True, '$HOME_NET')), ('src_port', (True, 'any')), ('arrow', '->'), \
        ('destination', (False, '$EXTERNAL_NET')), ('dst_port', (True, 'any'))])

            """

        if self.get_header():
            header = self.get_header()
            if re.search(r"[,\[\]]\s", header):
                header = re.sub(r",\s+", ",", header)
                header = re.sub(r"\s+,", ",", header)
                header = re.sub(r"\[\s+", "[", header)
                header = re.sub(r"\s+\]", "]", header)
            header = header.split()
        else:
            raise ValueError("Header is missing, or unparsable")
        # get rid of empty list elements
        header = list(filter(None, header))
        header_dict = collections.OrderedDict()
        size = len(header)
        if not size == 7 and not size == 1:
            msg = "Snort rule header is malformed %s" % header
            raise ValueError(msg)

        for item in header:
                if "action" not in header_dict:
                    action = self.actions(item)
                    header_dict["action"] = action
                    continue

                if "proto" not in header_dict:
                    try:
                        proto = self.proto(item)
                        header_dict["proto"] = proto
                        continue
                    except Exception as perror:
                        raise ValueError(perror)

                if "source" not in header_dict:
                    try:
                        src_ip = self.ip(item)
                        header_dict["source"] = src_ip
                        continue
                    except Exception as serror:
                        raise ValueError(serror)

                if "src_port" not in header_dict:
                    src_port = self.port(item)
                    header_dict["src_port"] = src_port
                    continue

                if "arrow" not in header_dict:
                    dst = self.destination(item)
                    header_dict["arrow"] = dst
                    continue

                if "destination" not in header_dict:
                    dst_ip = self.ip(item)
                    header_dict["destination"] = dst_ip
                    continue

                if "dst_port" not in header_dict:
                    dst_port = self.port(item)
                    header_dict["dst_port"] = dst_port
                    continue

        return header_dict

    def parse_options(self, rule=None):
        if rule:
            self.rule = rule
        opts = self.get_options()

        options_dict = collections.OrderedDict()
        for index, option_string in enumerate(opts):
            if ':' in option_string:
                negate = ""
                if option_string.strip().startswith("!"):
                    option_string = option_string.strip()[1:]
                    negate = "!"
                option = option_string.split(":", 1)
                key, value = option
                if key != "pcre":
                    value = value.split(",")
                options_dict[index] = (key, value)
            else:
                options_dict[index] = (option_string, negate + "")
        return options_dict

    def validate_options(self, options):

        for index, option in options.items():
            key, value = option
            if len(value) == 1:
                content_mod = self.dicts.content_modifiers(value[0])
                opt = False
                if content_mod:
                    # An unfinished feature
                    continue
            gen_option = self.dicts.options(key)
            if gen_option:
                opt = True
                continue
            pay_option = self.dicts.options(key)
            if pay_option:
                opt = True
                continue
            non_pay_option = self.dicts.options(key)
            if non_pay_option:
                opt = True
                continue
            post_detect = self.dicts.options(key)
            if post_detect:
                opt = True
                continue
            threshold = self.dicts.options(key)
            if threshold:
                opt = True
                continue
            if not opt:
                message = "unrecognized option: %s" % key
                raise ValueError(message)
        return options


class Sanitizer(object):
    def __init__(self):
        self.methods = {
                "pcre": self.pcre,
                # "depth": self.depth
                }

    def sanitize(self, parsed):
        options = parsed['options']
        for key, value in options.items():
            if key in self.methods:
                options[key] = self.methods[key](value)

        parsed['options'] = options
        return parsed

    @staticmethod
    def pcre(value: list) -> List:
        value_string = value[0]
        if re.match(r'^"/.*/[ismxAEGRUBPHMCOIDKYS]+"$', value_string):
            return value
        else:
            if not str(value).startswith('\"/') and value:
                start = re.split(r'^"', value)
                start[0] = '"/'
                value = ''.join(start)
            if not re.search(r'(\/")$', value):
                end = re.split(r'"$', value)
                end[-1] = '/"'
                value = ''.join(end)
            return value

    def depth(self, options):
        depth_idx = [idx for idx in options if "depth" in options[idx]][0]
        dsize_idx = [idx for idx in options if "dsize" in options[idx]][0]
        depth = options[depth_idx].get("depth")[0]
        dsize = options[dsize_idx].get("dsize")[0]
        full_dsize = re.split(r'[0-9]+', dsize)
        operand = [x for x in full_dsize if x]
        dsize = dsize.strip(operand[0])
        if int(depth) < int(dsize):
            return dsize
        else:
            return depth


class SerializeRule(object):

    def __init__(self, rule):
        self.rule = rule

    def __getitem__(self, key):
        if "rule" in key:
            return self.serialize_rule()
        if "header" in key:
            return self.serialize_header()
        if "options" in key:
            return self.serialize_options()

    def __str__(self):
        return self.serialize_rule()

    def __list_serializer(self, list_bool: bool, items: List) -> str:
        serialised = str()
        for _bool, item in items:
            if isinstance(item, list):
                serialised = "{},{}".format(serialised, self.__list_serializer(_bool, item))
            else:
                if _bool:
                    serialised = "{},{}".format(serialised, item)
                if not _bool:
                    serialised = "{},!{}".format(serialised, item)

        serialised_list = serialised.lstrip(",")

        if list_bool:
            serialised = "[{}]".format(serialised_list)
        else:
            serialised = "![{}]".format(serialised_list)

        return serialised

    def serialize_header_item(self, item: Any) -> str:
        if isinstance(item, str):
            return item

        if isinstance(item, tuple):
            _bool, item = item
            if isinstance(item, list):
                return self.__list_serializer(_bool, item)
            else:
                return item

    def serialize_header(self, header: Dict = None) -> str:
        serialised = str()
        if not header:
            header = self.rule['header']
        for key, value in header.items():
            item = self.serialize_header_item(value)
            serialised = '{} {}'.format(serialised, item)
        return serialised

    def serialize_options(self, options: Dict = None) -> str:
        options_list = []
        if not options:
            options = self.rule['options']
        for index, option in options.items():
            key, value = option
            if value:
                option_value = "{}:{}".format(key, ",".join(value))
            else:
                option_value = "{}".format(key)
            options_list.append(option_value)

        _options = '; '.join(str(e) for e in options_list)
        serialized_options = '({})'.format(_options)
        return serialized_options

    def serialize_rule(self):
        return "{} {}".format(self.serialize_header(), self.serialize_options()).lstrip()


if __name__ == '__main__':
    try:
        rule = Parser("alert tcp 192.168.0.1 any -> $HOME_NET 21 (msg:”FTP connection attempt”; sid:1000002; rev:1;)")
        print(rule.rule + " is OK")
    except Exception as e:
        print(e)


from typing import Any


class Dicts():

    def classtypes(self, cltype):
        classtypes = {
                "attempted-admin": "Attempted Administrator Privilege Gain",
                "attempted-dos": "Attempted Denial of Service",
                "attempted-recon": "Attempted Information Leak",
                "attempted-user": "Attempted User Privilege Gain",
                "bad-unknown": "Potentially Bad Traffic",
                "client-side-exploit": "Known client side exploit attempt",
                "default-login-attempt": "Attempt to Login By a Default Username and Password",
                "denial-of-service": "Detection of a Denial of Service Attack",
                "file-format": "Known malicious file or file based exploit",
                "icmp-event": "Generic ICMP Event",
                "inappropriate-content": "Inappropriate content was detected",
                "malware-cnc": "Known malware command and control traffic",
                "misc-activity": "Misc Activity",
                "misc-attack": "Misc Attack",
                "network-scan": "Detection of a Network Scan",
                "non-standard-protocol": "Detection of a Non-Standard Protocol or Event",
                "not-suspicious": "Not Suspicious Traffic",
                "policy-violation": "Potential Corporate Policy Violation",
                "protocol-command-decode": "Generic Protocol Command Decode",
                "rpc-portmap-decode": " Decode of an RPC Query",
                "sdf": "Sensitive Data",
                "shellcode-detect": "Executable Code was Detected",
                "string-detect": "A Suspicious String was Detected",
                "successful-admin": "Successful Administrator Privilege Gain",
                "successful-dos": "Denial of Service",
                "successful-recon-largescale": "Large Scale Information Leak",
                "successful-recon-limited": "Information Leak",
                "successful-user": "Successful User Privilege Gain",
                "suspicious-filename-detect": "A Suspicious Filename was Detected",
                "suspicious-login": "An Attempted Login Using a Suspicious Username was Detected",
                "system-call-detect": "A System Call was Detected",
                "tcp-connection": "A TCP Connection was Detected",
                "trojan-activity": "A Network Trojan was Detected",
                "unknown": "Unknown Traffic",
                "unsuccessful-user": "Unsuccessful User Privilege Gain",
                "unusual-client-port-connection": "A Client was Using an Unusual Port",
                "web-application-activity": "Access to a Potentially Vulnerable Web Application",
                "web-application-attack": "Web Application Attack",
                "nonstd-tcp": "Detection of a Non-Standard TCP Protocol"
                }
        if cltype in classtypes:
            return classtypes[cltype]
        else:
            return False

    @staticmethod
    def ip_variables(variable):
        variables = {"$EXTERNAL_NET": "$EXTERNAL_NET",
                     "$HTTP_SERVERS": "$HTTP_SERVERS",
                     "$INTERNAL_NET": "$INTERNAL_NET",
                     "$SQL_SERVERS": "$SQL_SERVERS",
                     "$SMTP_SERVERS": "$SMTP_SERVERS",
                     "$DNS_SERVERS": "$DNS_SERVERS",
                     "$HOME_NET": "$HOME_NET",
                     "any": "any"
                     }

        if variable in variables:
            return variables[variable]
        else:
            return False

    @staticmethod
    def general_options(option: str = None) -> Any:
        # TODO: maybe add Snort Default Classifications
        general_options = {"msg",
                           # The msg keyword tells the
                           # logging and alerting engine
                           # the message to print with the packet
                           # dump or alert.
                           "reference",
                           # The reference keyword allows rules to include
                           # references to external attack identification
                           # systems.
                           "gid",
                           # The gid keyword (generator id) is used to
                           # identify  what part of Snort generates the event
                           # when a particular rule fires.
                           "sid",
                           # The sid keyword is used to uniquely identify
                           # Snort rules.
                           "rev",
                           # The rev keyword is used to uniquely identify
                           # revisions of Snort rules.
                           "classtype",
                           # The classtype keyword is used to categorize
                           # a rule as detecting  an attack that is part
                           # of a more general type of attack class.
                           "priority",
                           # The priority keyword assigns a severity level to
                           # rules. "priority": "priority",
                           "metadata"
                           # The metadata keyword allows a rule writer
                           # to embed additional  information about the rule,
                           # typically in a key-value format.  Keys: engine
                           # ( Indicate a Shared Library Rule ) ex: "shared",
                           # soid ( Shared Library Rule Generator and
                           # SID ) ex: "gid|sid", service ( Target-Based
                           # Service Identifier ) ex: "http"
                           }

        if option:
            if option in general_options:
                return option
            else:
                return False
        else:
            return general_options

    @staticmethod
    def payload_detection(option: str = None) -> Any:

        payload_detection = {"content",
                             # The content keyword allows
                             # the user to set rules that search for specific
                             # content in the packet payload and trigger
                             # response based on that data.
                             "protected_content",
                             # As with the content keyword,
                             # its primary purpose is to match strings of
                             # specific bytes. The search is performed by
                             # hashing portions of incoming packets and
                             # comparing the results against the hash provided,
                             # and as such, it is computationally expensive.
                             "hash",
                             # The hash keyword is used to specify the hashing
                             # algorithm to use when matching a
                             # protected_content rule.
                             "length",
                             # The length keyword is used to specify the
                             # original length of the content specified
                             # in a protected_content rule digest.
                             # The value provided must be greater than 0 and
                             # less than 65536.
                             "nocase",
                             # The nocase keyword allows the rule writer to
                             # specify that the Snort should look for the
                             # specific pattern, ignoring case. nocase
                             # modifies the previous content keyword
                             # in the rule.
                             "rawbytes",
                             # The rawbytes keyword allows rules to look at
                             # the raw packet data, ignoring any decoding
                             # that was done by preprocessors.
                             "depth",
                             # The depth keyword allows the rule writer to
                             # specify how far into a packet Snort should
                             # search for the specified pattern.
                             "offset",
                             # The offset keyword allows the rule writer to
                             # specify where to start searching for a pattern
                             # within a packet.
                             "distance",
                             # The distance keyword allows the rule writer to
                             # specify how far into a packet Snort should
                             # ignore before starting to search for the
                             # specified pattern relative to the end of the
                             # previous pattern match.
                             "within",
                             # The within keyword is a content modifier that
                             # makes sure that at most N bytes are between
                             # pattern matches using the content keyword.
                             # NOTE: The http_client_body modifier is not
                             # allowed to be used with the rawbytes modifier
                             # for the same content.
                             "http_client_body",
                             # The http_client_body keyword is a content
                             # modifier that restricts the search to the body
                             # of an HTTP client request.
                             # NOTE: The http_cookie modifier is not
                             # allowed to be used with the
                             # rawbytes or fast_pattern modifiers for the same
                             # content
                             "http_cookie",
                             # The http_cookie keyword is a content modifier
                             # that restricts the search to the extracted
                             # Cookie Header field As this keyword
                             # is a modifier to the previous content
                             # keyword, there must be a content in the rule
                             # before http_cookie is specified.
                             # This keyword is dependent
                             # on the enable_cookie config option.
                             # NOTE: The http_raw_cookie modifier
                             # is not allowed to be used with the rawbytes,
                             # http_cookie or fast_pattern modifiers for the
                             # same content
                             "http_raw_cookie",
                             # The http_raw_cookie keyword is a content
                             # modifier that restricts the search to the
                             # extracted UNNORMALIZED Cookie Header field
                             # NOTE: The http_header modifier is not allowed
                             # to be used with the rawbytes modifier for the
                             # same content.
                             "http_header",
                             # The http_header keyword is a content modifier
                             # that restricts the search to the extracted
                             # Header fields.
                             # NOTE:The http_raw_header modifier
                             # is not allowed to be used with the rawbytes,
                             # http_header or fast_pattern modifiers for the
                             # same content.
                             "http_raw_header",
                             # The http_raw_header keyword is a content
                             # modifier that restricts the search to the
                             # extracted UNNORMALIZED Header fields
                             # NOTE: The http_method modifier
                             # is not allowed to be used with the rawbytes
                             # or fast_pattern
                             # modifiers for the same content.
                             "http_method",
                             # The http_method keyword is a content modifier
                             # that restricts the search to the extracted
                             # Method from a HTTP client request.
                             # NOTE: The http_uri modifier is not allowed
                             # to be used with the rawbytes modifier for
                             # the same content.
                             "http_uri",
                             # The http_uri keyword is a content modifier
                             # that restricts the search to the NORMALIZED
                             # request URI field. NOTE: The http_raw_uri
                             # modifier is not allowed to be used with the
                             # rawbytes, http_uri or fast_pattern modifiers
                             # for the same content.
                             "http_raw_uri",
                             # The http_raw_uri keyword is a content modifie
                             # that restricts the search to the UNNORMALIZED
                             # request URI field. NOTE: The http_stat_code
                             # modifier is not allowedi to be used with the
                             # rawbytes or fast_pattern modifiers for the
                             # same content.
                             "http_stat_code",
                             # The http_stat_code keyword is a content
                             # modifier that restricts the search
                             # to the extracted Status code field
                             # from a HTTP server response.
                             # NOTE: The http_stat_msg modifier is not allowed
                             # to be used with the rawbytes or fast_pattern
                             # modifiers for the same content.
                             "http_stat_msg",
                             # The http_stat_msg keyword is a content modifier
                             # that restricts the search to the extracted
                             # Status Message field from a
                             # HTTP server response.
                             # NOTE: Negation(!) and OR(|) operations cannot
                             # be used in conjunction with each other for the
                             # http_encode keyword. The OR and negation
                             # operations work only on the encoding type
                             # field and not on http buffer type field.
                             # TODO: check for http_encode options
                             "http_encode",
                             # The http_encode keyword will enable alerting
                             # based on encoding type present in a HTTP client
                             # request or a HTTP server response
                             # NOTE: The fast_pattern modifier cannot be used
                             # with the following http content modifiers:
                             # 1. http_cookie,
                             # 2. http_raw_uri,
                             # 3. http_raw_header,
                             # 4. http_raw_cookie,
                             # 5. http_method,
                             # 6. http_stat_code,
                             # 7. http_stat_msg
                             # NOTE: The fast_pattern modifier can be used
                             # with negated contents onlyi if those contents
                             # are not modified with:
                             # 1. offset,
                             # 2. depth,
                             # 3. distance or
                             # 4.  within.
                             # NOTE: The fast pattern matcher is always case
                             # insensitive. TODO: check for fast_pattern
                             # format
                             "fast_pattern",
                             # The fast_pattern keyword is a content modifier
                             # that sets the content within a rule to be used
                             # with the fast pattern matcher.
                             # NOTE: uricontent cannot be modified by
                             # a rawbytes modifier or any of the other
                             # HTTP modifiers. If you wish to search the
                             # UNNORMALIZED request URI field, use the
                             # http_raw_uri modifier with a content option.
                             "uricontent",
                             # The uricontent keyword in the Snort
                             # rule language searches the normalized request
                             # URI field.
                             "urilen",
                             # The urilen keyword in the Snort rule
                             # language specifies the exact length,
                             # the minimum length, the maximum length,
                             # or range of URI lengths to match.
                             "isdataat",
                             # The isdataat keyword verifies that the
                             # payload has data at a specified location.
                             # TODO: check for Perl compatible modifiers
                             # for pcre. NOTE: Since this is an advanced
                             # option, check the manual for pitfalls.
                             "pcre",
                             # The pcre keyword allows rules to
                             # be written using perl compatible regular
                             # expressions.
                             "pkt_data",
                             # This option sets the cursor used for detection
                             # to the raw transport payload.
                             # NOTE: The argument mime to file_data is
                             # deprecated. The rule options file_data will
                             # itself point to the decoded MIME attachment.
                             "file_data",
                             # This option sets the cursor used for detection
                             # to one of the following buffers:
                             # 1. HTTP response body
                             # 2. HTTP de-chunked response body
                             # 3. HTTP decompressed response
                             # 4. HTTP normalized response body
                             # 5. HTTP UTF normalized response body
                             # 6. All of the above
                             # 7. SMTP/POP/IMAP data body
                             # 8. Base64 decoded MIME attachment
                             # 9. Non-Encoded MIME attachment
                             # 10. Quoted-Printable decoded MIME attachment
                             # 11. Unix-to-Unix decoded attachment
                             # TODO: check for base64_decode options and
                             # format
                             "base64_decode",
                             # This option is used to decode the
                             # base64 encoded data. This option is
                             # particularly useful in
                             # case of HTTP headers such as HTTP authorization
                             # headers. NOTE: Fast pattern content matches
                             # are not allowed with this buffer.
                             "base64_data",
                             # This option is similar to the rule option
                             # file_data and is used to set the
                             # cursor used for detection to the beginning
                             # of the base64 decoded
                             # buffer if present.
                             # TODO: check for options.
                             "byte_test",
                             # The byte_test keyword tests a byte
                             # field against a specific value
                             # (with operatori). TODO: check
                             # for options
                             "byte_jump",
                             # The byte_jump keyword allows rules to read the
                             # length of a portion of data, then skip that
                             # far forward in the packet. NOTE: Only two
                             # byte_extract variables may be created per rule.
                             # They can be re-used in the same rule any number
                             # of times. TODO. check for options.
                             "byte_extract",
                             # It reads in some number of bytes from the
                             # packet payload and saves it to a variable.
                             # TODO: check for byte_math syntax and options
                             "byte_math",
                             # Perform a mathematical
                             # operation on an extracted
                             # value and a specified value or
                             # existing variable,
                             # and store the outcome in a new resulting
                             # variable
                             "ftpbounce",
                             # The ftpbounce keyword detects FTP bounce
                             # attacks. TODO. check for options and syntax
                             # for asn1
                             "asn1",
                             # The asn1 detection plugin decodes a packet or a
                             # portion of a packet, and looks for variou
                             #  malicious encodings.
                             # NOTE: This plugin cannot do detection over
                             # encrypted sessions, e.g. SSH (usually port 22).
                             # TODO: find a way to check if the rule uses
                             # encrypted sessions
                             "cvs",
                             # The cvs keyword detects invalid entry strings.
                             "dce_iface",
                             # For DCE/RPC based rules it has been necessary
                             # to set flow-bits based on a client bind to a
                             # service to avoid false positives.
                             "dce_opnum",
                             # The opnum represents a specific function
                             # call to an interface.
                             "dce_stub_data",
                             # This option is used to place the cursor
                             # (used to walk the packet payload in rules
                             # processing) at the beginning of the DCE/RPC
                             # stub data SIP Preprocessor provides ways to
                             # tackle Common Vulnerabilities and Exposures
                             # (CVEs) related with SIP found over the past
                             # few years.
                             "sip_method",
                             # The sip_method keyword is used to check for
                             # specific SIP request methods.
                             "sip_stat_code",
                             # The sip_stat_code is used to check the SIP
                             # response status code.
                             # This option matches if any one of the state
                             # codes specified matches the status codes of
                             # the SIP response.
                             "sip_header",
                             # The sip_header keyword restricts the search
                             # to the extracted Header fields of a SIP message
                             # request or a response. This works similar to
                             # file_data.
                             "sip_body",
                             # The sip_body keyword places the cursor at the
                             # beginning of the Body fields of a SIP message.
                             # This works similar to file_data and
                             # dce_stub_data. The message body includes
                             # channel information using SDP protocol
                             # (Session Description Protocol).
                             # GTP (GPRS Tunneling Protocol) is used in core
                             # communication networks to establish a channel
                             # between GSNs (GPRS Serving Node). GTP decoding
                             # preprocessor provides ways to tackle
                             # intrusion attempts to those networks through
                             # GTP. It also makes detecting new attacks easier.
                             # TODO: identify also gtp message types, but for
                             # now keyword check has to cut it.
                             "gtp_type",
                             # The gtp_type keyword is used to check for
                             # specific GTP types. User can input message type
                             # value, an integer in [0, 255], or a string
                             # defined in the Table below.
                             # TODO: gtp_info table check.
                             "gtp_info",
                             # The gtp_info keyword is used to check for
                             # specific GTP information element.
                             # This keyword restricts the search to the
                             # information element field. User can input
                             # information element value,
                             # an integer in $[0, 255]$,
                             "gtp_version"
                             # The gtp_version keyword is used to check for
                             # specific GTP version. Relates to gtp_info
                             # and gtp_type tables.
                             }

        if option:
            if option in payload_detection:
                return option
            else:
                return False
        else:
            return payload_detection

    @staticmethod
    def non_payload_options(option: str = None) -> Any:
        non_payload_detect = {"fragoffset",
                              # The fragoffset keyword allows one to compare
                              # the IP fragment offset field against a
                              # decimal value.
                              "ttl",
                              # The ttl keyword is used to check the IP
                              # time-to-live value.
                              "tos",
                              # The tos keyword is used to check the IP
                              # TOS field for a specific value.
                              "id",
                              # The id keyword is used to check the IP ID
                              # field for a specific value.
                              "ipopts",
                              # The ipopts keyword is used to check if a
                              # specific IP option is present.
                              "fragbits",
                              # The fragbits keyword is used to check if
                              # fragmentation and reserved bits are set
                              # in the IP header.
                              "dsize",
                              # The dsize keyword is used to test the
                              # packet payload isize NOTE: The reserved bits
                              # '1' and '2' have been replaced with
                              # 'C' and 'E',respectively, to match
                              # RFC 3168, "The Addition of Explicit
                              # Congestion Notification (ECN) to IP".
                              # The old values of '1' and '2' are still
                              # valid for the flag keyword, but are now
                              # deprecated.
                              "flags",
                              # The flags keyword is used to check if
                              # specific TCP flag bits are present.
                              # TODO: check for syntax and options
                              "flow",
                              # The flow keyword allows rules to only
                              # apply to certain directions of the traffic
                              # flow. TODO. check for options and syntax
                              "flowbits",
                              # The flowbits keyword allows rules to
                              # track states during a transport protocol
                              # session.
                              "seq",
                              # The seq keyword is used to check for a
                              # specific TCP sequence number
                              "ack",
                              # The ack keyword is used to check for a
                              # specific TCP acknowledge number
                              "window",
                              # The window keyword is used to check for
                              # a specific TCP window size
                              "itype",
                              # The itype keyword is used to check for a
                              # specific ICMP type value
                              "icode",
                              # The icode keyword is used to check for
                              # a specific ICMP code value
                              "icmp_id",
                              # The icmp id keyword is used to check
                              # for a specific ICMP ID value.
                              "icmp_seq",
                              # The icmp seq keyword is used to check
                              # for a specific ICMP sequence value.
                              "rpc",
                              # The rpc keyword is used to check
                              # for a RPC application, version, and
                              # procedure numbers in SUNRPC CALL requests
                              "ip_proto",
                              # The ip proto keyword allows checks against
                              # the IP protocol header
                              "sameip",
                              # The sameip keyword allows rules to check
                              # if the source ip is the same as the
                              # destination IP.
                              # NOTE: The stream_reassemble option is
                              # only available when the Stream preprocessor
                              # is enabled.
                              "stream_reassemble",
                              # The stream_reassemble keyword allows a rule
                              # to enable or disable TCP stream
                              # reassembly on matching traffic.
                              # NOTE: The stream_size option is only
                              # available when the Stream preprocessor
                              # is enabled.
                              "stream_size"
                              # The stream_size keyword allows a rule
                              # to match traffic
                              # according to the number of bytes observed,
                              # as determined by the TCP sequence numbers.
                              }

        if option:
            if option in non_payload_detect:
                return option
            else:
                return False
        else:
            return non_payload_detect

    @staticmethod
    def post_detect_options(option: str = None) -> Any:
        post_detect = {"logto",
                       # The logto keyword tells Snort to log all packets
                       # that trigger this rule to a special output log file.
                       "session",
                       # The session keyword is built to extract user data
                       # from TCP Sessions
                       "resp",
                       # The resp keyword is used attempt to close sessions
                       # when an alert is triggered.
                       "react",
                       # This keyword implements an ability for users to
                       # react to traffic that matches a Snort rule by
                       # closing connection and sending a noticei.
                       # NOTE: also check for options
                       "tag",
                       # The tag keyword allow rules to log more than
                       # just the single packet that triggered the rule
                       "replace",
                       # Replace the prior matching content with the given
                       # string of the same length. Available in inline
                       # mode only. NOTE: As mentioned above, Snort evaluates
                       #  detection_filter as the last step of the detection
                       # and not in post-detection.
                       "activates",
                       # This keyword allows the rule writer to specify
                       # a rule to add when a specific network event occurs.
                       "activated_by",
                       # This keyword allows the rule writer to dynamically
                       # enable a rule when a specific activate rule is
                       # triggered.
                       "count",
                       # This keyword must be used in combination with the
                       # activated by keyword. It
                       # allows the rule writer to specify how many packets
                       # to leave the rule enabled for
                       # after it is activated
                       "detection_filter",
                       # Replace the prior matching content with the given
                       # string of the same length. Available
                       # in inline mode only.
                       }

        if option:
            if option in post_detect:
                return option
            else:
                return False
        else:
            return post_detect

    @staticmethod
    def content_modifiers(option: str = None) -> Any:
        content_modifiers = {"nocase",
                             "rawbytes",
                             "depth",
                             "offset",
                             "distance",
                             "within",
                             "http_client_body",
                             "http_cookie",
                             "http_raw_cookie",
                             "http_header",
                             "http_raw_header",
                             "http_method",
                             "http_uri",
                             "http_raw_uri",
                             "http_stat_code",
                             "http_stat_msg",
                             "http_encode",
                             "fast_pattern",
                             "uricontent",
                             "urilen",
                             "isdataat",
                             "pcre",
                             "pkt_data",
                             "file_data",
                             "base64_decode",
                             "base64_data",
                             "byte_test",
                             "byte_jump",
                             "byte_extract",
                             "byte_math",
                             "ftpbounce",
                             "asn1",
                             "cvs",
                             "dce_iface",
                             "dce_opnum",
                             "dce_stub_data",
                             "sip_method",
                             "sip_stat_code",
                             "sip_header",
                             "sip_body",
                             "gtp_type",
                             "gtp_info",
                             "gtp_version",
                             "ssl_version",
                             "ssl_state"
                             }

        if option:
            if option in content_modifiers:
                return option
            else:
                return False
        else:
            return content_modifiers

    @staticmethod
    def rule_thresholds(option):

        threshold = {"threshold"}

        if option in threshold:
            return option
        else:
            return False

    def options(self, option):

        # TODO: maybe add Snort Default Classifications
        general_options = self.general_options()
        payload_detection = self.payload_detection()
        content_modifiers = self.content_modifiers()
        non_payload_detect = self.non_payload_options()
        post_detect = self.post_detect_options()


        # TODO: add threshold types ex: threshold:
        # type limit <<, but for now, this will have to suffice
        rule_tresholds = {"threshold": "threshold"}

        # check if rule is of payload detect type
        if option in payload_detection:
            return "payload", option
        if option in non_payload_detect:
            return "non-payload", option
        if option in general_options:
            return "general", option
        if option in rule_tresholds:
            return "threshold", option
        if option in content_modifiers:
            return "content_modifier", option
        if option in post_detect:
            return "post_detect", option

    def get_options(self):
        options = set()
        return options.union(
            self.general_options(),
            self.payload_detection(),
            self.content_modifiers(),
            self.non_payload_options(),
            self.post_detect_options()
        )


# Generated by Django 4.1.2 on 2023-01-05 14:35

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('snort', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SnortRuleViewArray',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False)),
                ('typeOfItem', models.CharField(blank=True, max_length=12)),
                ('locationX', models.IntegerField()),
                ('locationY', models.IntegerField()),
                ('value', models.CharField(blank=True, max_length=256)),
                ('snortId', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='snort.snortrule')),
            ],
        ),
    ]



# Generated by Django 4.1.2 on 2022-12-19 18:20

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='attackGroup',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('description', models.TextField(max_length=256)),
            ],
        ),
        migrations.CreateModel(
            name='keywords',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='Name of site-wide variable', max_length=200)),
                ('type', models.CharField(blank=True, help_text='Value of site-wide variable that scripts can reference - must be valid JSON', max_length=100, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Setting',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='Name of site-wide variable', max_length=200)),
                ('value', models.CharField(blank=True, help_text='Value of site-wide variable that scripts can reference - must be valid JSON', max_length=100, null=True)),
            ],
        ),
    ]



# Generated by Django 4.1.2 on 2022-12-26 06:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='keywords',
            name='type',
        ),
        migrations.AddField(
            model_name='keywords',
            name='description',
            field=models.CharField(blank=True, help_text='Value of site-wide variable that scripts can reference', max_length=100, null=True),
        ),
        migrations.AddField(
            model_name='keywords',
            name='negation',
            field=models.CharField(blank=True, help_text='Value of site-wide variable that scripts can reference', max_length=100, null=True),
        ),
        migrations.AddField(
            model_name='keywords',
            name='options',
            field=models.CharField(blank=True, help_text='Value of site-wide variable that scripts can reference', max_length=100, null=True),
        ),
        migrations.AddField(
            model_name='keywords',
            name='stage',
            field=models.CharField(blank=True, help_text='Value of site-wide variable that scripts can reference', max_length=100, null=True),
        ),
    ]




from setuptools import setup, find_packages


setup(
    name='snort_web_master',
    version='0.2.0.0',
    license='MoCorp',
    author="meir dahan",
    author_email='1dahanmeir1@gmail.com',
    packages=find_packages('snort_web_master'),
    package_dir={'': 'snort_web_master'},
    url='https://github.com/mosheovadi1/snort-web-master',
    keywords='snort3 django',
    include_package_data=True,
    install_requires=[
          'django',
          'django-auth-ldap3-ad',
          'django-object-actions',
          'dpkt',
          'django-advanced-filters',
          'suricataparser',
          'django-import-export',
          "psycopg2-binary==2.9.1",
          "gunicorn==20.1.0"
      ],

)


# Generated by Django 4.1.2 on 2023-01-08 18:38

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0003_keywords_avalable'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='keywords',
            name='negation',
        ),
    ]



from django.contrib import admin
from settings.models import Setting
from settings.models import attackGroup
from settings.models import keywords
# Register your models here.

@admin.register(Setting)
class SettingAdmin(admin.ModelAdmin):
    list_display = ['name', 'value']

@admin.register(attackGroup)
class attack_groupAdmin(admin.ModelAdmin):
    list_display = ['name', 'description']


@admin.register(keywords)
class keywords(admin.ModelAdmin):
    list_display = ['name', 'stage', "options", "description"]
